@@ -177,7 +177,7 @@ uploadPackageTarball <- function(package, repoName, repoPath, ...) {
 
   # Annotate the package DESCRIPTION with the repository
   tmpTarballPath <- file.path(tempdir(), "packrat-tarball-upload")
-  untar(package, exdir = tmpTarballPath, tar = "internal")
+  untar(package, exdir = tmpTarballPath, tar = tar_binary())
   pkgName <- sub("_.*", "", basename(package))
   untarredPath <- file.path(tmpTarballPath, pkgName)
   setRepositoryField(
@@ -189,7 +189,13 @@ uploadPackageTarball <- function(package, repoName, repoPath, ...) {
   setwd(tmpTarballPath)
   on.exit(setwd(owd), add = TRUE)
 
-  success <- tar(basename(package), files = pkgName)
+  success <- tar(
+    basename(package),
+    files = pkgName,
+    compression = "gzip",
+    tar = tar_binary()
+  )
+
   if (success != 0)
     stop("Failed to re-tar package tarball")
 
