@@ -66,7 +66,7 @@ install <- function(pkg = ".", reload = TRUE, quick = FALSE, local = TRUE,
   }
   opts <- paste(paste(opts, collapse = " "), paste(args, collapse = " "))
 
-  R(paste("CMD INSTALL ", shQuote(built_path), " ", opts, sep = ""),
+  R(paste("CMD INSTALL --preclean ", shQuote(built_path), " ", opts, sep = ""),
     quiet = quiet)
 
   if (reload) reload(pkg$package, quiet = quiet)
@@ -89,7 +89,7 @@ build <- function(pkg = ".", path = NULL, binary = FALSE, vignettes = TRUE,
 
   if (binary) {
     args <- c("--build", args)
-    cmd <- paste0("CMD INSTALL ", shQuote(pkg$path), " ",
+    cmd <- paste0("CMD INSTALL --preclean ", shQuote(pkg$path), " ",
                   paste0(args, collapse = " "))
     ext <- if (.Platform$OS.type == "windows") "zip" else "tgz"
   } else {
@@ -116,7 +116,7 @@ build <- function(pkg = ".", path = NULL, binary = FALSE, vignettes = TRUE,
 
 R.path <- function() file.path(R.home("bin"), "R")
 
-R <- function(options, path = tempdir(), env_vars = NULL, ...) {
+R <- function(options, path = tempdir(), ...) {
   options <- paste("--vanilla", options)
   r_path <- file.path(R.home("bin"), "R")
 
@@ -126,11 +126,19 @@ R <- function(options, path = tempdir(), env_vars = NULL, ...) {
     on.exit(set_path(old))
   }
 
-  in_dir(path, system_check(r_path, options, c(r_env_vars(), env_vars), ...))
+  in_dir(
+    path,
+    system_check(
+      cmd = r_path,
+      args = options,
+      env = c(r_env_vars(), envvar_mask()),
+      ...
+    )
+  )
 }
 
 r_env_vars <- function() {
-  c("LC_ALL" = "C",
+  c(
     "R_LIBS" = paste(getLibPaths(), collapse = .Platform$path.sep),
     "CYGWIN" = "nodosfilewarning",
     # When R CMD check runs tests, it sets R_TESTS. When the tests
@@ -139,16 +147,40 @@ r_env_vars <- function() {
     # the R subprocesses. Unsetting it here avoids those problems.
     "R_TESTS" = "",
     "NOT_CRAN" = "true",
-    "TAR" = auto_tar())
+    "TAR" = tar_binary()
+  )
 }
 
-auto_tar <- function() {
-  tar <- Sys.getenv("TAR", unset = NA)
-  if (!is.na(tar)) return(tar)
+envvar_mask <- function() {
+  # Mask tokens unless told not to.
+  git_token_vars <- if (getOption("packrat.mask.git.service.envvars", TRUE)) {
+    c(
+      "GITHUB_PAT",
+      "GITLAB_PAT",
+      "BITBUCKET_USERNAME",
+      "BITBUCKET_USER",
+      "BITBUCKET_PASSWORD",
+      "BITBUCKET_PASS",
+      # Varnames that may have been used previously
+      "GITHUB_USERNAME",
+      "GITHUB_USER",
+      "GITHUB_PASSWORD",
+      "GITHUB_PASS",
+      "GITLAB_USERNAME",
+      "GITLAB_USER",
+      "GITLAB_PASSWORD",
+      "GITLAB_PASS"
+    )
+  } else {
+    NULL
+  }
 
-  windows <- .Platform$OS.type == "windows"
-  no_rtools <- is.null(get_rtools_path())
-  if (windows && no_rtools) "internal" else ""
+  user_specified_vars <- getOption("packrat.masked.envvars", NULL)
+  all_vars <- c(git_token_vars, user_specified_vars)
+  envvar_mask <- as.character(rep(NA, length(all_vars)))
+  names(envvar_mask) <- all_vars
+
+  return(envvar_mask)
 }
 
 with_something <- function(set) {
@@ -169,6 +201,14 @@ set_libpaths <- function(paths) {
 
 with_libpaths <- with_something(set_libpaths)
 
+# Modifies environment variables, executes the code in `code` and then restores
+# the environment variables to their prior values.
+# - `new` should be a named character vector of values for environment variables
+#   to take during execution. Variables can be temporarily unset with an `NA`
+#   value.
+# - `action` can be "prefix" or "suffix" to combine `new` with existing
+#   variables instead of replacing.
+# See `set_envvar` for more details.
 with_envvar <- function(new, code, action = "replace") {
   old <- set_envvar(new, action)
   on.exit(set_envvar(old, "replace"))
@@ -187,8 +227,10 @@ install_local_path_single <- function(path, subdir = NULL, before_install = NULL
 
   if (!file.info(path)$isdir) {
     bundle <- path
-    path <- decompress(path)
-    on.exit(unlink(path), add = TRUE)
+    target <- tempfile("packrat-install-")
+    dir.create(target)
+    path <- decompress(path, target)
+    on.exit(unlink(target, recursive = TRUE), add = TRUE)
   } else {
     bundle <- NULL
   }
@@ -234,7 +276,7 @@ with_build_tools <- function(code) {
 
 decompress <- function(src, target = tempdir()) {
   tryCatch(
-    decompressImpl(src, target),
+    suppressWarnings(decompressImpl(src, target)),
     error = function(e) {
       fmt <- paste(
         "Failed to extract archive:",
@@ -251,27 +293,21 @@ decompress <- function(src, target = tempdir()) {
 decompressImpl <- function(src, target = tempdir()) {
   stopifnot(file.exists(src))
 
-  # force internal tar (otherwise bad things can happen on Windows if
-  # unexpected versions of tar.exe are on the PATH)
-  TAR <- Sys.getenv("TAR")
-  Sys.setenv(TAR = "internal")
-  on.exit(Sys.setenv(TAR = TAR), add = TRUE)
-
   if (grepl("\\.zip$", src)) {
     unzip(src, exdir = target, unzip = getOption("unzip"))
     outdir <- getrootdir(as.vector(unzip(src, list = TRUE)$Name))
 
   } else if (grepl("\\.tar$", src)) {
-    untar(src, exdir = target)
-    outdir <- getrootdir(untar(src, list = TRUE))
+    untar(src, exdir = target, tar = tar_binary())
+    outdir <- getrootdir(untar(src, list = TRUE, tar = tar_binary()))
 
   } else if (grepl("\\.(tar\\.gz|tgz)$", src)) {
-    untar(src, exdir = target, compressed = "gzip")
-    outdir <- getrootdir(untar(src, compressed = "gzip", list = TRUE))
+    untar(src, exdir = target, compressed = "gzip", tar = tar_binary())
+    outdir <- getrootdir(untar(src, compressed = "gzip", list = TRUE, tar = tar_binary()))
 
   } else if (grepl("\\.(tar\\.bz2|tbz)$", src)) {
-    untar(src, exdir = target, compressed = "bzip2")
-    outdir <- getrootdir(untar(src, compressed = "bzip2", list = TRUE))
+    untar(src, exdir = target, compressed = "bzip2", tar = tar_binary())
+    outdir <- getrootdir(untar(src, compressed = "bzip2", list = TRUE, tar = tar_binary()))
 
   } else {
     ext <- gsub("^[^.]*\\.", "", src)
@@ -464,7 +500,7 @@ scan_registry_for_rtools <- function(debug = FALSE) {
   for (i in seq_along(keys)) {
     version <- names(keys)[[i]]
     key <- keys[[version]]
-    if (!is.list(key) || is.null(key$InstallPath)) next;
+    if (!is.list(key) || is.null(key$InstallPath)) next
     install_path <- normalizePath(key$InstallPath,
                                   mustWork = FALSE, winslash = "/")
 
@@ -518,7 +554,7 @@ rtools <- function(path, version) {
 }
 is.rtools <- function(x) inherits(x, "rtools")
 
-rtools_url <- "http://cran.r-project.org/bin/windows/Rtools/"
+rtools_url <- "https://cran.r-project.org/bin/windows/Rtools/"
 version_info <- list(
   "2.11" = list(
     version_min = "2.10.0",
@@ -575,10 +611,10 @@ rtools_needed <- function() {
 }
 
 system_check <- function(cmd, args = character(), env = character(),
-                         quiet = FALSE, ...) {
+                         quiet = FALSE, return_output = FALSE, ...) {
   full <- paste(shQuote(cmd), paste(args, collapse = ", "))
 
-  if (!quiet) {
+  if (!quiet && !return_output) {
     message(wrap_command(full))
     message()
   }
@@ -587,7 +623,7 @@ system_check <- function(cmd, args = character(), env = character(),
   # on Windows
   result <- suppressWarnings(with_envvar(
     env,
-    if (quiet) {
+    if (quiet || return_output) {
       system2(cmd, args, stdout = TRUE, stderr = TRUE)
     } else {
       system2(cmd, args)
@@ -618,6 +654,10 @@ system_check <- function(cmd, args = character(), env = character(),
     stop(stopMsg, call. = FALSE)
   }
 
+  if (return_output) {
+    return(result)
+  }
+
   invisible(TRUE)
 }
 
@@ -628,8 +668,16 @@ wrap_command <- function(x) {
   paste(lines, continue, collapse = "\n")
 }
 
+# `set_envvar` takes a named character vector and sets its contents to update
+# environment variables. It returns the old value of the modified envvars.
+# - All non-NA entries in the list will be written to the environment. The
+#   default action overwrites the environment variables, but "prefix" and
+#   "suffix" combine the new value with the existing value.
+# - Any names in the list with NA values will be unset using `Sys.unsetenv`
+# `with_envvar` uses this function to temporarily replace environment variables
+# for execution of a code block.
 set_envvar <- function(envs, action = "replace") {
-  stopifnot(is.named(envs))
+  stopifnot(all.named(envs))
   stopifnot(is.character(action), length(action) == 1)
   action <- match.arg(action, c("replace", "prefix", "suffix"))
 
@@ -651,7 +699,7 @@ set_envvar <- function(envs, action = "replace") {
   invisible(old)
 }
 
-is.named <- function(x) {
+all.named <- function(x) {
   !is.null(names(x)) && all(names(x) != "")
 }
 
