@@ -14,7 +14,8 @@
 #' packages your project depends on.
 #' \item \strong{Reproducible}: Packrat records the exact package versions you
 #' depend on, and ensures those exact versions are the ones that get installed
-#' wherever you go.}
+#' wherever you go.
+#' }
 #'
 #' Use \code{\link{init}} to create a new packrat project,
 #' \code{\link{snapshot}} to record changes to your project's library, and
@@ -70,7 +71,7 @@
 #' @docType package
 #' @name packrat
 #' @import utils
-#' @author RStudio, Inc.
+#' @author Posit Software, PBC
 NULL
 
 #' Initialize Packrat on a new or existing \R project
@@ -220,15 +221,18 @@ initImpl <- function(project = getwd(),
   augmentRprofile(project)
   options <- initOptions(project, opts) ## writes out packrat.opts and returns generated list
 
-  # Take a snapshot
-  snapshotImpl(project,
-               lib.loc = NULL,
-               ignore.stale = TRUE,
-               fallback.ok = TRUE,
-               infer.dependencies = infer.dependencies)
+  # If we don't yet have a lockfile, take a snapshot and then build the Packrat library.
+  if (!file.exists(lockFilePath(project = project))) {
 
-  # Use the lockfile to copy sources and install packages to the library
-  restore(project, overwrite.dirty = TRUE, restart = FALSE)
+    snapshotImpl(project,
+                 lib.loc = NULL,
+                 ignore.stale = TRUE,
+                 fallback.ok = TRUE,
+                 infer.dependencies = infer.dependencies)
+
+    restore(project, overwrite.dirty = TRUE, restart = FALSE)
+
+  }
 
   # Copy init.R so a user can 'start from zero' with a project
   file.copy(
@@ -251,7 +255,7 @@ initImpl <- function(project = getwd(),
 
     # Restart R if the environment is capable of it (otherwise enter packrat mode)
     if (!restart || !attemptRestart())
-      on(project = project, clean.search.path = TRUE)
+      on(project = project, clean.search.path = FALSE)
   }
 
   invisible()
@@ -266,14 +270,12 @@ initImpl <- function(project = getwd(),
 #'
 #' There are three common use cases for \code{restore}:
 #' \itemize{
-#'   \item \strong{Hydrate}: Use \code{restore} after copying a project to a new
-#' machine to populate the library on that machine.
-#'
-#'   \item \strong{Sync}: Use \code{restore} to apply library changes made by a
+#' \item \strong{Hydrate}: Use \code{restore} after copying a project to a new machine
+#' to populate the library on that machine.
+#' \item \strong{Sync}: Use \code{restore} to apply library changes made by a
 #' collaborator to your own library. (In general, you want to run \code{restore}
 #' whenever you pick up a change to \code{packrat.lock})
-#'
-#'   \item \strong{Rollback}: Use \code{restore} to undo accidental changes made
+#' \item \strong{Rollback}: Use \code{restore} to undo accidental changes made
 #' to the library since the last snapshot.
 #' }
 #'
@@ -281,44 +283,64 @@ initImpl <- function(project = getwd(),
 #' changes are necessary to currently loaded packages, you will need to restart
 #' \R to apply the changes (\code{restore} will let you know when this is
 #' necessary). It is recommended that you do this as soon as possible, because
-#' any library changes made between running \code{restore} and restarting \R will
-#' be lost.
+#' any library changes made between running \code{restore} and restarting \R
+#' will be lost.
 #'
-#' @note
-#' \code{restore} can be destructive; it will remove packages that were not in
-#' the snapshot, and it will replace newer packages with older versions if
-#' that's what the snapshot indicates. \code{restore} will warn you before
-#' attempting to remove or downgrade a package (if \code{prompt} is
-#' \code{TRUE}), but will always perform upgrades and new installations without
-#' prompting.
+#' @note \code{restore} can be destructive; it will remove packages that were
+#'   not in the snapshot, and it will replace newer packages with older versions
+#'   if that's what the snapshot indicates. \code{restore} will warn you before
+#'   attempting to remove or downgrade a package (if \code{prompt} is
+#'   \code{TRUE}), but will always perform upgrades and new installations
+#'   without prompting.
 #'
-#' \code{restore} works only on the private package library created by packrat;
-#' if you have other libraries on your path, they will be unaffected.
+#'   \code{restore} works only on the private package library created by
+#'   packrat; if you have other libraries on your path, they will be unaffected.
 #'
-#' The \code{restart} parameter will only result in a restart of R when the
-#' R environment packrat is running within makes available a restart function
-#' via \code{getOption("restart")}.
+#'   The \code{restart} parameter will only result in a restart of R when the R
+#'   environment packrat is running within makes available a restart function
+#'   via \code{getOption("restart")}.
 #'
-#' @param project The project directory. When in packrat mode, if this is \code{NULL},
-#' then the directory associated with the current packrat project is used. Otherwise,
-#' the project directory specified is used.
+#'   To install packages hosted in private repositories on GitHub, GitLab, and
+#'   Bitbucket, you must either set the option
+#'   \code{packrat.authenticated.downloads.use.renv} to \code{TRUE} and ensure
+#'   that \code{curl} is available on your system, or ensure that the
+#'   \code{httr} package is available in your R library.
+#'
+#'   In addition, you must make credentials for your provider available in the
+#'   appropriate environment variable(s): \code{GITHUB_PAT}, \code{GITLAB_PAT},
+#'   and/or \code{BITBUCKET_USERNAME} and \code{BITBUCKET_PASSWORD}. These
+#'   environment variables are hidden from package installation subprocesses.
+#'
+#'   Packrat does not support installation from enterprise instances of GitHub,
+#'   GitLab, or Bitbucket.
+#'
+#'   Packrat selects a \code{tar} binary with the following heuristic: If a
+#'   \code{TAR} environment variable exists, Packrat will use that. Otherwise,
+#'   it will either look for a \code{tar} binary on the \code{PATH} on Unix, or
+#'   look for the system \code{tar} on Windows. If no binary is found in those
+#'   locations, it will use R's internal \code{tar} implementation, which may
+#'   cause errors with long filenames.
+#'
+#' @param project The project directory. When in packrat mode, if this is
+#'   \code{NULL}, then the directory associated with the current packrat project
+#'   is used. Otherwise, the project directory specified is used.
 #' @param overwrite.dirty A dirty package is one that has been changed since the
-#' last snapshot or restore. Packrat will leave these alone by default. If you
-#' want to guarantee that \code{restore} will put you in the exact state
-#' represented by the snapshot being applied, use \code{overwrite.dirty = TRUE}.
+#'   last snapshot or restore. Packrat will leave these alone by default. If you
+#'   want to guarantee that \code{restore} will put you in the exact state
+#'   represented by the snapshot being applied, use \code{overwrite.dirty =
+#'   TRUE}.
 #' @param prompt \code{TRUE} to prompt before performing potentially destructive
-#' changes (package removals or downgrades); \code{FALSE} to perform these
-#' operations without confirmation.
+#'   changes (package removals or downgrades); \code{FALSE} to perform these
+#'   operations without confirmation.
 #' @param dry.run If \code{TRUE}, compute the changes to your packrat state that
 #'   would be made if a restore was performed, without actually executing them.
 #' @param restart If \code{TRUE}, restart the R session after restoring.
 #'
-#' @seealso
-#' \code{\link{snapshot}}, the command that creates the snapshots applied with
-#' \code{restore}.
+#' @seealso \code{\link{snapshot}}, the command that creates the snapshots
+#'   applied with \code{restore}.
 #'
-#' \code{\link{status}} to view the differences between the most recent snapshot
-#' and the library.
+#'   \code{\link{status}} to view the differences between the most recent
+#'   snapshot and the library.
 #'
 #' @export
 restore <- function(project = NULL,
@@ -328,7 +350,7 @@ restore <- function(project = NULL,
                     restart = !dry.run) {
 
   project <- getProjectDir(project)
-  stopIfNotPackified(project)
+  stopIfNoLockfile(project)
 
   if (!dry.run) {
     callHook(project, "restore", TRUE)
@@ -402,7 +424,7 @@ restore <- function(project = NULL,
     options(repos = externalRepos)
   }, add = TRUE)
 
-  # Install each package from CRAN or github, from binaries when available and
+  # Install each package from CRAN or github/bitbucket/gitlab, from binaries when available and
   # then from sources.
   restoreImpl(project, repos, packages, libDir,
               pkgsToIgnore = pkgsToIgnore, prompt = prompt,
@@ -445,7 +467,6 @@ clean <- function(packages = NULL,
                   force = FALSE) {
 
   project <- getProjectDir(project)
-  stopIfNotPackified(project)
 
   callHook(project, "clean", TRUE)
   on.exit(callHook(project, "clean", FALSE), add = TRUE)
@@ -520,8 +541,8 @@ unused_packages <- function(project = NULL,
   orphans <- setdiff(installedPkgNames,
                      packagesInUse)
 
-  ## Exclude 'manipulate', 'rstudio'
-  orphans <- setdiff(orphans, c("manipulate", "rstudio"))
+  ## Exclude 'manipulate', 'rstudio', and ignored packages
+  orphans <- setdiff(orphans, c("manipulate", "rstudio", opts$ignored.packages()))
   orphanRecs <- getPackageRecords(orphans,
                                   project = project,
                                   available = NULL,
@@ -580,7 +601,7 @@ packify <- function(project = NULL, quiet = FALSE) {
   invisible()
 }
 
-lockInfo <- function(project, property='packages', fatal=TRUE) {
+lockInfo <- function(project, property = 'packages', fatal = TRUE) {
 
   project <- getProjectDir(project)
 
